@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////
 //                           **** WAVPACK ****                            //
 //                  Hybrid Lossless Wavefile Compressor                   //
-//                Copyright (c) 1998 - 2022 David Bryant.                 //
+//                Copyright (c) 1998 - 2026 David Bryant.                 //
 //                          All Rights Reserved.                          //
 //      Distributed under the BSD Software License (see license.txt)      //
 ////////////////////////////////////////////////////////////////////////////
@@ -61,7 +61,7 @@
 
 static const char *sign_on = "\n"
 " WVGAIN  ReplayGain Scanner/Tagger for WavPack  %s Version %s\n"
-" Copyright (c) 2005 - 2022 David Bryant.  All Rights Reserved.\n\n";
+" Copyright (c) 2005 - 2026 David Bryant.  All Rights Reserved.\n\n";
 
 static const char *version_warning = "\n"
 " WARNING: WVGAIN using libwavpack version %s, expected %s (see README)\n\n";
@@ -84,6 +84,10 @@ static const char *usage =
 "                 info if album mode specified)\n"
 "          -q  = quiet (keep console output to a minimum)\n"
 "          -s  = show stored values only (no analysis)\n"
+#ifdef ENABLE_THREADS
+"          -t  = use multiple threads for faster operation\n"
+"          -1  = don't use multiple threads for faster operation\n"
+#endif
 "          -v  = write the version to stdout\n"
 #if defined (_WIN32)
 "          -z  = don't set console title to indicate progress\n\n"
@@ -101,7 +105,7 @@ int debug_logging_mode;
 static uint32_t track_histogram [HISTOGRAM_SLOTS], album_histogram [HISTOGRAM_SLOTS];
 
 static char album_mode, clean_mode, display_mode, ignore_wvc, quiet_mode, show_mode, new_mode, set_console_title;
-static int num_files, file_index;
+static int num_files, file_index, worker_threads;
 
 /////////////////////////// local function declarations ///////////////////////
 
@@ -118,10 +122,6 @@ static void display_progress (double file_progress);
 static void TextToUTF8 (void *string, int len);
 #endif
 
-#define WAVPACK_NO_ERROR    0
-#define WAVPACK_SOFT_ERROR  1
-#define WAVPACK_HARD_ERROR  2
-
 // The "main" function for the command-line WavPack ReplayGain Scanner/Processor.
 // Note that on Windows this is actually a static function that is called from the
 // "real" main() defined immediately afterward that converts the wchar argument list
@@ -166,6 +166,10 @@ int main(int argc, char **argv)
    set_console_title = 1;      // on Windows, we default to messing with the console title
 #endif                          // on Linux, this is considered uncool to do by default
 
+#ifdef ENABLE_THREADS
+    worker_threads = get_default_worker_threads ();
+#endif
+
     // loop through command-line arguments
 
     while (--argc) {
@@ -224,6 +228,18 @@ int main(int argc, char **argv)
                         show_mode = 1;
                         break;
 
+                    case 'T': case 't':
+#ifdef ENABLE_THREADS
+                        worker_threads = 4;
+#else
+                        error_line ("warning: threads not enabled, ignoring -t option!");
+#endif
+                        break;
+
+                    case '1':
+                        worker_threads = 0;
+                        break;
+
                     default:
                         error_line ("illegal option: %c !", **argv);
                         ++error_count;
@@ -582,7 +598,7 @@ static void float_samples (float *dst, int32_t *src, uint32_t samcnt, float scal
 static int analyze_file (char *infilename, uint32_t *histogram, float *peak)
 {
     int result = WAVPACK_NO_ERROR, open_flags = 0, num_channels, wvc_mode;
-    uint32_t sample_rate, window_samples;
+    uint32_t sample_rate, window_samples, num_windows = 1;
     int64_t total_unpacked_samples = 0;
     void *decimation_context = NULL;
     double progress = -1.0;
@@ -603,6 +619,7 @@ static int analyze_file (char *infilename, uint32_t *histogram, float *peak)
         open_flags |= OPEN_WVC;
 
     open_flags |= OPEN_TAGS | OPEN_NORMALIZE | OPEN_DSD_AS_PCM;
+    open_flags |= worker_threads << OPEN_THREADS_SHFT;
 
     wpc = WavpackOpenFileInput (infilename, error, open_flags, 0);
 
@@ -634,19 +651,22 @@ static int analyze_file (char *infilename, uint32_t *histogram, float *peak)
 
     window_samples = sample_rate / 20;
 
+    if (worker_threads)
+        num_windows = (worker_threads + 1) * 48000 / window_samples;
+
     if (decimation_context)
-        temp_buffer = malloc (window_samples * 8 * 4);
+        temp_buffer = malloc (num_windows * window_samples * 8 * 4);
     else
-        temp_buffer = malloc (window_samples * 8);
+        temp_buffer = malloc (num_windows * window_samples * 8);
 
     if (!filter_init (sample_rate))
         result = WAVPACK_SOFT_ERROR;
 
     while (result == WAVPACK_NO_ERROR) {
         uint32_t samples_to_unpack, samples_unpacked;
-        int32_t level;
+        int32_t *window_ptr = temp_buffer, level;
 
-        samples_to_unpack = window_samples;
+        samples_to_unpack = num_windows * window_samples;
 
         if (decimation_context) {
             samples_unpacked = WavpackUnpackSamples (wpc, temp_buffer, samples_to_unpack * 4);
@@ -658,40 +678,45 @@ static int analyze_file (char *infilename, uint32_t *histogram, float *peak)
             total_unpacked_samples += samples_unpacked;
         }
 
-        if (samples_unpacked) {
+        if (!samples_unpacked)
+            break;
+
+        if (num_channels == 1) {
+            int32_t *dst = temp_buffer + samples_unpacked * 2;
+            int32_t *src = temp_buffer + samples_unpacked;
+            uint32_t cnt = samples_unpacked;
+
+            while (cnt--) {
+                *--dst = *--src;
+                *--dst = *src;
+            }
+        }
+
+        while (samples_unpacked) {
+            int samples_this_window = samples_unpacked > window_samples ? window_samples : samples_unpacked;
+
             if (!(WavpackGetMode (wpc) & MODE_FLOAT))
                 switch (WavpackGetBytesPerSample (wpc)) {
                     case 1:
-                        float_samples ((float *) temp_buffer, temp_buffer, samples_unpacked * num_channels, 1.0 / 128.0);
+                        float_samples ((float *) window_ptr, window_ptr, samples_this_window * 2, 1.0 / 128.0);
                         break;
 
                     case 2:
-                        float_samples ((float *) temp_buffer, temp_buffer, samples_unpacked * num_channels, 1.0 / 32768.0);
+                        float_samples ((float *) window_ptr, window_ptr, samples_this_window * 2, 1.0 / 32768.0);
                         break;
 
                     case 3:
-                        float_samples ((float *) temp_buffer, temp_buffer, samples_unpacked * num_channels, 1.0 / 8388608.0);
+                        float_samples ((float *) window_ptr, window_ptr, samples_this_window * 2, 1.0 / 8388608.0);
                         break;
 
                     case 4:
-                        float_samples ((float *) temp_buffer, temp_buffer, samples_unpacked * num_channels, 1.0 / 2147483648.0);
+                        float_samples ((float *) window_ptr, window_ptr, samples_this_window * 2, 1.0 / 2147483648.0);
                         break;
                 }
 
-            if (num_channels == 1) {
-                int32_t *dst = temp_buffer + samples_unpacked * 2;
-                int32_t *src = temp_buffer + samples_unpacked;
-                uint32_t cnt = samples_unpacked;
-
-                while (cnt--) {
-                    *--dst = *--src;
-                    *--dst = *src;
-                }
-            }
-
-            calc_stereo_peak ((float *) temp_buffer, samples_unpacked, peak);
-            filter_stereo_samples ((float *) temp_buffer, samples_unpacked);
-            level = (int32_t) floor (100 * calc_stereo_rms ((float *) temp_buffer, samples_unpacked));
+            calc_stereo_peak ((float *) window_ptr, samples_this_window, peak);
+            filter_stereo_samples ((float *) window_ptr, samples_this_window);
+            level = (int32_t) floor (100 * calc_stereo_rms ((float *) window_ptr, samples_this_window));
 
             if (level < 0)
                 histogram [0]++;
@@ -699,9 +724,10 @@ static int analyze_file (char *infilename, uint32_t *histogram, float *peak)
                 histogram [HISTOGRAM_SLOTS - 1]++;
             else
                 histogram [level]++;
+
+            window_ptr += samples_this_window * 2;
+            samples_unpacked -= samples_this_window;
         }
-        else
-            break;
 
         if (check_break ()) {
 #if defined(_WIN32)
