@@ -216,6 +216,20 @@ int64_t WavpackGetFileSize64 (WavpackContext *wpc)
     return wpc ? wpc->filelen + wpc->file2len : 0;
 }
 
+// Get the actual number of total samples, or if writing a file this returns the number of
+// samples written so far. This allows WavpackGetRatio() and WavpackGetAverageBitrate() to
+// work during writing and specifically in cases where the initial length was unknown.
+
+static int64_t actual_total_samples (WavpackContext *wpc)
+{
+    int64_t total_samples = wpc->total_samples;
+
+    if (wpc->wv_out && wpc->streams && wpc->streams [0] && wpc->streams [0]->sample_index)
+        total_samples = wpc->streams [0]->sample_index;
+
+    return total_samples;
+}
+
 // Calculate the ratio of the specified WavPack file size to the size of the
 // original audio data as a double greater than 0.0 and (usually) smaller than
 // 1.0. A value greater than 1.0 represents "negative" compression and a
@@ -223,8 +237,8 @@ int64_t WavpackGetFileSize64 (WavpackContext *wpc)
 
 double WavpackGetRatio (WavpackContext *wpc)
 {
-    if (wpc && wpc->total_samples != -1 && wpc->filelen) {
-        double output_size = (double) wpc->total_samples * wpc->config.num_channels *
+    if (wpc && actual_total_samples (wpc) != -1 && wpc->filelen) {
+        double output_size = (double) actual_total_samples (wpc) * wpc->config.num_channels *
             wpc->config.bytes_per_sample;
         double input_size = (double) wpc->filelen + wpc->file2len;
 
@@ -241,8 +255,8 @@ double WavpackGetRatio (WavpackContext *wpc)
 
 double WavpackGetAverageBitrate (WavpackContext *wpc, int count_wvc)
 {
-    if (wpc && wpc->total_samples != -1 && wpc->filelen && WavpackGetSampleRate (wpc)) {
-        double output_time = (double) wpc->total_samples / WavpackGetSampleRate (wpc);
+    if (wpc && actual_total_samples (wpc) != -1 && wpc->filelen && WavpackGetSampleRate (wpc)) {
+        double output_time = (double) actual_total_samples (wpc) / WavpackGetSampleRate (wpc);
         double input_size = (double) wpc->filelen + (count_wvc ? wpc->file2len : 0);
 
         if (output_time >= 0.1 && input_size >= 1.0)
@@ -346,6 +360,31 @@ void WavpackGetChannelIdentities (WavpackContext *wpc, unsigned char *identities
     *identities = 0;
 }
 
+#ifdef ENABLE_THREADS
+
+static void worker_threads_destroy (WavpackContext *wpc)
+{
+    if (wpc->workers) {
+        int i;
+
+        for (i = 0; i < wpc->num_workers; ++i) {
+            wp_mutex_obtain (wpc->mutex);
+            wpc->workers [i].state = Quit;
+            wp_condvar_signal (wpc->workers [i].worker_cond);
+            wp_mutex_release (wpc->mutex);
+            wp_thread_join (wpc->workers [i].thread);
+            wp_thread_delete (wpc->workers [i].thread);
+            wp_condvar_delete (wpc->workers [i].worker_cond);
+        }
+
+        free (wpc->workers);
+        wpc->workers = NULL;
+        wp_mutex_delete (wpc->mutex);
+    }
+}
+
+#endif
+
 // For local use only. Install a callback to be executed when WavpackCloseFile() is called,
 // usually used to dump some statistics accumulated during encode or decode.
 
@@ -409,6 +448,10 @@ WavpackContext *WavpackCloseFile (WavpackContext *wpc)
         decimate_dsd_destroy (wpc->decimation_context);
 #endif
 
+#ifdef ENABLE_THREADS
+    worker_threads_destroy (wpc);
+#endif
+
     free (wpc);
 
     return NULL;
@@ -530,29 +573,7 @@ void free_streams (WavpackContext *wpc)
     int si = wpc->num_streams;
 
     while (si--) {
-        if (wpc->streams [si]->blockbuff) {
-            free (wpc->streams [si]->blockbuff);
-            wpc->streams [si]->blockbuff = NULL;
-        }
-
-        if (wpc->streams [si]->block2buff) {
-            free (wpc->streams [si]->block2buff);
-            wpc->streams [si]->block2buff = NULL;
-        }
-
-        if (wpc->streams [si]->sample_buffer) {
-            free (wpc->streams [si]->sample_buffer);
-            wpc->streams [si]->sample_buffer = NULL;
-        }
-
-        if (wpc->streams [si]->dc.shaping_data) {
-            free (wpc->streams [si]->dc.shaping_data);
-            wpc->streams [si]->dc.shaping_data = NULL;
-        }
-
-#ifdef ENABLE_DSD
-        free_dsd_tables (wpc->streams [si]);
-#endif
+        free_single_stream (wpc->streams [si]);
 
         if (si) {
             wpc->num_streams--;
@@ -560,10 +581,44 @@ void free_streams (WavpackContext *wpc)
             wpc->streams [si] = NULL;
         }
     }
-
-    wpc->current_stream = 0;
 }
 
+// Free all resources associated with the specified stream
+
+void free_single_stream (WavpackStream *wps)
+{
+    if (wps->blockbuff) {
+        free (wps->blockbuff);
+        wps->blockbuff = NULL;
+    }
+
+    if (wps->block2buff) {
+        free (wps->block2buff);
+        wps->block2buff = NULL;
+    }
+
+    if (wps->sample_buffer) {
+        free (wps->sample_buffer);
+        wps->sample_buffer = NULL;
+    }
+
+    if (wps->pre_sample_buffer) {
+        free (wps->pre_sample_buffer);
+        wps->pre_sample_buffer = NULL;
+    }
+
+    if (wps->dc.shaping_data) {
+        free (wps->dc.shaping_data);
+        wps->dc.shaping_data = NULL;
+    }
+
+#ifdef ENABLE_DSD
+    free_dsd_tables (wps);
+#endif
+}
+
+// Free all DSD-related resources associated with the specified stream
+
 void free_dsd_tables (WavpackStream *wps)
 {
     if (wps->dsd.probabilities) {
