@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////
 //                           **** DSDPACK ****                            //
 //         Lossless DSD (Direct Stream Digital) Audio Compressor          //
-//                Copyright (c) 2013 - 2016 David Bryant.                 //
+//                Copyright (c) 2013 - 2024 David Bryant.                 //
 //                          All Rights Reserved.                          //
 //      Distributed under the BSD Software License (see license.txt)      //
 ////////////////////////////////////////////////////////////////////////////
@@ -27,10 +27,8 @@ static int init_dsd_block_high (WavpackStream *wps, WavpackMetadata *wpmd);
 static int decode_fast (WavpackStream *wps, int32_t *output, int sample_count);
 static int decode_high (WavpackStream *wps, int32_t *output, int sample_count);
 
-int init_dsd_block (WavpackContext *wpc, WavpackMetadata *wpmd)
+int init_dsd_block (WavpackStream *wps, WavpackMetadata *wpmd)
 {
-    WavpackStream *wps = wpc->streams [wpc->current_stream];
-
     if (wpmd->byte_length < 2)
         return FALSE;
 
@@ -40,7 +38,12 @@ int init_dsd_block (WavpackContext *wpc, WavpackMetadata *wpmd)
     if (*wps->dsd.byteptr > 31)
         return FALSE;
 
-    wpc->dsd_multiplier = 1U << *wps->dsd.byteptr++;
+    // safe to cast away const on stream 0 only
+    if (!wps->stream_index)
+        ((WavpackContext *)wps->wpc)->dsd_multiplier = 1U << *wps->dsd.byteptr++;
+    else
+        wps->dsd.byteptr++;
+
     wps->dsd.mode = *wps->dsd.byteptr++;
 
     if (!wps->dsd.mode) {
@@ -60,9 +63,8 @@ int init_dsd_block (WavpackContext *wpc, WavpackMetadata *wpmd)
         return FALSE;
 }
 
-int32_t unpack_dsd_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sample_count)
+int32_t unpack_dsd_samples (WavpackStream *wps, int32_t *buffer, uint32_t sample_count)
 {
-    WavpackStream *wps = wpc->streams [wpc->current_stream];
     uint32_t flags = wps->wphdr.flags;
 
     // don't attempt to decode past the end of the block, but watch out for overflow!
@@ -91,11 +93,18 @@ int32_t unpack_dsd_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sampl
         }
         else if (!decode_high (wps, buffer, sample_count))
             wps->mute_error = TRUE;
+
+        // If we just finished this block, then it's time to check if the applicable checksum matches. Like
+        // other decoding errors, this is indicated by setting the mute_error flag.
+
+        if (wps->sample_index + sample_count == GET_BLOCK_INDEX (wps->wphdr) + wps->wphdr.block_samples &&
+            !wps->mute_error && wps->crc != wps->wphdr.crc)
+                wps->mute_error = TRUE;
     }
 
     if (wps->mute_error) {
         int samples_to_null;
-        if (wpc->reduced_channels == 1 || wpc->config.num_channels == 1 || (flags & MONO_FLAG))
+        if (wps->wpc->reduced_channels == 1 || wps->wpc->config.num_channels == 1 || (flags & MONO_FLAG))
             samples_to_null = sample_count;
         else
             samples_to_null = sample_count * 2;
@@ -134,6 +143,8 @@ static int init_dsd_block_fast (WavpackStream *wps, WavpackMetadata *wpmd)
     unsigned char history_bits, max_probability, *lb_ptr;
     int total_summed_probabilities = 0, bi, i;
 
+    (void) wpmd;
+
     if (wps->dsd.byteptr == wps->dsd.endptr)
         return FALSE;
 
@@ -167,7 +178,7 @@ static int init_dsd_block_fast (WavpackStream *wps, WavpackMetadata *wpmd)
                     *outptr++ = 0;
             }
             else if (code)
-                *outptr++ = code;
+                *outptr++ = (unsigned char)code;
             else
                 break;
         }
@@ -186,7 +197,7 @@ static int init_dsd_block_fast (WavpackStream *wps, WavpackMetadata *wpmd)
         int32_t sum_values;
 
         for (sum_values = i = 0; i < 256; ++i)
-            wps->dsd.summed_probabilities [bi] [i] = sum_values += wps->dsd.probabilities [bi] [i];
+            wps->dsd.summed_probabilities [bi] [i] = (uint16_t)(sum_values += wps->dsd.probabilities [bi] [i]);
 
         if (sum_values) {
             if ((total_summed_probabilities += sum_values) > wps->dsd.history_bins * MAX_BYTES_PER_BIN)
@@ -198,7 +209,7 @@ static int init_dsd_block_fast (WavpackStream *wps, WavpackMetadata *wpmd)
                 int c = wps->dsd.probabilities [bi] [i];
 
                 while (c--)
-                    *lb_ptr++ = i;
+                    *lb_ptr++ = (unsigned char)i;
             }
         }
     }
@@ -249,7 +260,7 @@ static int decode_fast (WavpackStream *wps, int32_t *output, int sample_count)
         if (index >= wps->dsd.summed_probabilities [wps->dsd.p0] [255])
             return 0;
 
-        if ((*output++ = code = wps->dsd.value_lookup [wps->dsd.p0] [index]))
+        if ((*output++ = code = wps->dsd.value_lookup [wps->dsd.p0] [index]) != 0)
             wps->dsd.low += wps->dsd.summed_probabilities [wps->dsd.p0] [code-1] * mult;
 
         wps->dsd.high = wps->dsd.low + wps->dsd.probabilities [wps->dsd.p0] [code] * mult - 1;
@@ -288,7 +299,7 @@ static int decode_fast (WavpackStream *wps, int32_t *output, int sample_count)
 
 #define RATE_S 20
 
-static void init_ptable (int *table, int rate_i, int rate_s)
+static void init_ptable (int32_t *table, int rate_i, int rate_s)
 {
     int value = 0x808000, rate = rate_i << 8, c, i;
 
@@ -313,6 +324,8 @@ static int init_dsd_block_high (WavpackStream *wps, WavpackMetadata *wpmd)
     uint32_t flags = wps->wphdr.flags;
     int channel, rate_i, rate_s, i;
 
+    (void) wpmd;
+
     if (wps->dsd.endptr - wps->dsd.byteptr < ((flags & MONO_DATA) ? 13 : 20))
         return FALSE;
 
@@ -507,7 +520,7 @@ void *decimate_dsd_init (int num_channels)
 {
     DecimationContext *context = (DecimationContext *)malloc (sizeof (DecimationContext));
     double filter_sum = 0, filter_scale;
-    int skipped_terms, i, j;
+    int i, j;
 
     if (!context)
         return context;
@@ -525,9 +538,8 @@ void *decimate_dsd_init (int num_channels)
         filter_sum += decm_filter [i];
 
     filter_scale = ((1 << 23) - 1) / filter_sum * 16.0;
-    // fprintf (stderr, "convolution, %d terms, %f sum, %f scale\n", NUM_FILTER_TERMS, filter_sum, filter_scale);
 
-    for (skipped_terms = i = 0; i < NUM_FILTER_TERMS; ++i) {
+    for (i = 0; i < NUM_FILTER_TERMS; ++i) {
         int scaled_term = (int) floor (decm_filter [i] * filter_scale + 0.5);
 
         if (scaled_term) {
@@ -537,12 +549,8 @@ void *decimate_dsd_init (int num_channels)
                 else
                     context->conv_tables [i >> 3] [j] -= scaled_term;
         }
-        else
-            skipped_terms++;
     }
 
-    // fprintf (stderr, "%d terms skipped\n", skipped_terms);
-
     decimate_dsd_reset (context);
 
     return context;
@@ -586,7 +594,7 @@ void decimate_dsd_run (void *decimate_context, int32_t *samples, int num_samples
         sum += context->conv_tables [6] [sp->delay [6] = sp->delay [7]];
         sum += context->conv_tables [7] [sp->delay [7] = sp->delay [8]];
         sum += context->conv_tables [8] [sp->delay [8] = sp->delay [9]];
-        sum += context->conv_tables [9] [sp->delay [9] = *samptr];
+        sum += context->conv_tables [9] [sp->delay [9] = (unsigned char)*samptr];
 #elif (HISTORY_BYTES == 7)
         sum += context->conv_tables [0] [sp->delay [0] = sp->delay [1]];
         sum += context->conv_tables [1] [sp->delay [1] = sp->delay [2]];
@@ -594,14 +602,14 @@ void decimate_dsd_run (void *decimate_context, int32_t *samples, int num_samples
         sum += context->conv_tables [3] [sp->delay [3] = sp->delay [4]];
         sum += context->conv_tables [4] [sp->delay [4] = sp->delay [5]];
         sum += context->conv_tables [5] [sp->delay [5] = sp->delay [6]];
-        sum += context->conv_tables [6] [sp->delay [6] = *samptr];
+        sum += context->conv_tables [6] [sp->delay [6] = (unsigned char)*samptr];
 #else
         int i;
 
         for (i = 0; i < HISTORY_BYTES-1; ++i)
             sum += context->conv_tables [i] [sp->delay [i] = sp->delay [i+1]];
 
-        sum += context->conv_tables [i] [sp->delay [i] = *samptr];
+        sum += context->conv_tables [i] [sp->delay [i] = (unsigned char)*samptr];
 #endif
 
         *samptr++ = (sum + 8) >> 4;
@@ -643,7 +651,6 @@ static void extrapolate_pcm (int32_t *samples, int samples_to_extrapolate, int s
             float left_ratio = (samples_to_extrapolate + period / 2.0F) / period, right_ratio = (period / 2.0F) / period;
             int32_t *sam1 = samples + samples_to_extrapolate * num_channels, *sam2 = sam1 + period * num_channels;
             float ave1 = 0.0, ave2 = 0.0;
-            int i;
 
             for (i = 0; i < period; ++i) {
                 ave1 += (float) sam1 [i * num_channels] / period;
